<?php

/*
 * Copyright by SoftCreatR.dev.
 *
 * License: https://softcreatr.dev/license-terms
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 * The above copyright notice and this disclaimer notice shall be included in all
 * copies or substantial portions of the Software.
 */

namespace wcf\system\event\listener;

use wcf\system\database\exception\DatabaseQueryException;
use wcf\system\database\exception\DatabaseQueryExecutionException;
use wcf\system\exception\NamedUserException;
use wcf\system\exception\UserInputException;
use wcf\system\WCF;

class ScConversationFloodControlListener implements IParameterizedEventListener
{
    /**
     * @inheritDoc
     *
     * @throws DatabaseQueryException
     * @throws NamedUserException
     * @throws UserInputException
     * @throws DatabaseQueryExecutionException
     */
    public function execute($eventObj, $className, $eventName, array &$parameters)
    {
        if ($eventName === 'initializeAction' && WCF::getSession()->getPermission('user.conversation.floodControlAnswer')) {
            $sql = 'SELECT time FROM wcf' . WCF_N . '_conversation_message WHERE userID = ? AND time > ? ORDER BY time DESC, messageID DESC';
            $statement = WCF::getDB()->prepareStatement($sql, 1);
            $statement->execute([
                WCF::getUser()->userID,
                TIME_NOW - WCF::getSession()->getPermission('user.conversation.floodControlAnswer')
            ]);

            if (($row = $statement->fetchArray()) !== false) {
                throw new NamedUserException(WCF::getLanguage()->getDynamicVariable(
                    'wcf.conversation.error.scFloodControlAnswer',
                    ['lastConversationTime' => $row['time']]
                ));
            }
        } elseif ($eventName === 'validate' && WCF::getSession()->getPermission('user.conversation.floodControlStart')) {
            $sql = 'SELECT time FROM wcf' . WCF_N . '_conversation WHERE userID = ? AND time > ? ORDER BY time DESC, conversationID DESC';
            $statement = WCF::getDB()->prepareStatement($sql, 1);
            $statement->execute([
                WCF::getUser()->userID,
                TIME_NOW - WCF::getSession()->getPermission('user.conversation.floodControlStart')
            ]);

            if (($row = $statement->fetchArray()) !== false) {
                WCF::getTPL()->assign(['lastConversationTime' => $row['time']]);

                throw new UserInputException('scFloodControlStart');
            }
        }
    }
}
